@@ -19,6 +19,11 @@
 #include "qgis_core.h"
 #include "qgsgeometry.h"
 #include "qgsvectorlayerfeatureiterator.h"
+#include "qgsfeaturesink.h"
+
+class QgsFeatureRenderer;
+class QgsSymbolLayer;
+class QgsSymbolLayerId;
 
 /**
  * \ingroup core
@@ -105,8 +110,8 @@ class CORE_EXPORT QgsVectorLayerUtils
      * Create a feature iterator for a specified field name or expression.
      * \param layer vector layer to retrieve values from
      * \param fieldOrExpression field name or an expression string
-     * \param ok will be set to false if field or expression is invalid, otherwise true
-     * \param selectedOnly set to true to get values from selected features only
+     * \param ok will be set to FALSE if field or expression is invalid, otherwise TRUE
+     * \param selectedOnly set to TRUE to get values from selected features only
      * \returns feature iterator
      * \since QGIS 3.0
      */
@@ -116,8 +121,8 @@ class CORE_EXPORT QgsVectorLayerUtils
      * Fetches all values from a specified field name or expression.
      * \param layer vector layer to retrieve values from
      * \param fieldOrExpression field name or an expression string
-     * \param ok will be set to false if field or expression is invalid, otherwise true
-     * \param selectedOnly set to true to get values from selected features only
+     * \param ok will be set to FALSE if field or expression is invalid, otherwise TRUE
+     * \param selectedOnly set to TRUE to get values from selected features only
      * \param feedback optional feedback object to allow cancellation
      * \returns list of fetched values
      * \see getDoubleValues
@@ -130,8 +135,8 @@ class CORE_EXPORT QgsVectorLayerUtils
      * invalid expression results are skipped.
      * \param layer vector layer to retrieve values from
      * \param fieldOrExpression field name or an expression string evaluating to a double value
-     * \param ok will be set to false if field or expression is invalid, otherwise true
-     * \param selectedOnly set to true to get values from selected features only
+     * \param ok will be set to FALSE if field or expression is invalid, otherwise TRUE
+     * \param selectedOnly set to TRUE to get values from selected features only
      * \param nullCount optional pointer to integer to store number of null values encountered in
      * \param feedback optional feedback object to allow cancellation
      * \returns list of fetched values
@@ -141,7 +146,7 @@ class CORE_EXPORT QgsVectorLayerUtils
     static QList< double > getDoubleValues( const QgsVectorLayer *layer, const QString &fieldOrExpression, bool &ok, bool selectedOnly = false, int *nullCount = nullptr, QgsFeedback *feedback = nullptr );
 
     /**
-     * Returns true if the specified value already exists within a field. This method can be used to test for uniqueness
+     * Returns TRUE if the specified value already exists within a field. This method can be used to test for uniqueness
      * of values inside a layer's attributes. An optional list of ignored feature IDs can be provided, if so, any features
      * with IDs within this list are ignored when testing for existence of the value.
      * \see createUniqueValue()
@@ -165,7 +170,7 @@ class CORE_EXPORT QgsVectorLayerUtils
 
     /**
      * Tests an attribute value to check whether it passes all constraints which are present on the corresponding field.
-     * Returns true if the attribute value is valid for the field. Any constraint failures will be reported in the errors argument.
+     * Returns TRUE if the attribute value is valid for the field. Any constraint failures will be reported in the errors argument.
      * If the strength or origin parameter is set then only constraints with a matching strength/origin will be checked.
      */
     static bool validateAttribute( const QgsVectorLayer *layer, const QgsFeature &feature, int attributeIndex, QStringList &errors SIP_OUT,
@@ -200,11 +205,14 @@ class CORE_EXPORT QgsVectorLayerUtils
      * Duplicates a feature and it's children (one level deep). It calls CreateFeature, so
      * default values and constraints (e.g., unique constraints) will automatically be handled.
      * The duplicated feature will be automatically inserted into the layer.
-     * \a depth the higher this number the deeper the level - With depth > 0 the children of the feature are not duplicated
      * \a duplicateFeatureContext stores all the layers and the featureids of the duplicated features (incl. children)
+     * \a maxDepth the maximum depth to duplicate children in relations, 0 is unlimited depth (in any case, limited to 100)
+     * \a depth the current depth, not exposed in Python
+     * \a referencedLayersBranch the current branch of layers across the relations, not exposed in Python, taken by copy not reference, used to avoid infinite loop
      * \since QGIS 3.0
      */
-    static QgsFeature duplicateFeature( QgsVectorLayer *layer, const QgsFeature &feature, QgsProject *project, int depth, QgsDuplicateFeatureContext &duplicateFeatureContext SIP_OUT );
+    static QgsFeature duplicateFeature( QgsVectorLayer *layer, const QgsFeature &feature, QgsProject *project, QgsDuplicateFeatureContext &duplicateFeatureContext SIP_OUT, const int maxDepth = 0, int depth SIP_PYARGREMOVE = 0, QList<QgsVectorLayer *> referencedLayersBranch SIP_PYARGREMOVE = QList<QgsVectorLayer *>() );
+
 
     /**
      * Gets the feature source from a QgsVectorLayer pointer.
@@ -212,9 +220,9 @@ class CORE_EXPORT QgsVectorLayerUtils
      * thread is safe too.
      * This should be used in scenarios, where a ``QWeakPointer<QgsVectorLayer>`` is kept in a thread
      * and features should be fetched from this layer. Using the layer directly is not safe to do.
-     * The result will be ``nullptr`` if the layer has been deleted.
+     * The result will be ``NULLPTR`` if the layer has been deleted.
      * If \a feedback is specified, the call will return if the feedback is canceled.
-     * Returns a new feature source for the \a layer. The source may be a nullptr if the layer no longer
+     * Returns a new feature source for the \a layer. The source may be NULLPTR if the layer no longer
      * exists or if the feedback is canceled.
      *
      * \note Requires Qt >= 5.10 to make use of the thread-safe implementation
@@ -251,17 +259,20 @@ class CORE_EXPORT QgsVectorLayerUtils
      * converting a multi part geometry to single part
      *
      * The following operations will be performed to convert the input features:
-     *  - convert single geometries to multi part
-     *  - drop additional attributes
-     *  - drop geometry if layer is geometry-less
-     *  - add missing attribute fields
-     *  - add back M/Z values (initialized to 0)
-     *  - drop Z/M
-     *  - convert multi part geometries to single part
+     *
+     * - convert single geometries to multi part
+     * - drop additional attributes
+     * - drop geometry if layer is geometry-less
+     * - add missing attribute fields
+     * - add back M/Z values (initialized to 0)
+     * - drop Z/M
+     * - convert multi part geometries to single part
+     *
+     * Optionally, \a sinkFlags can be specified to further refine the compatibility logic.
      *
      * \since QGIS 3.4
      */
-    static QgsFeatureList makeFeatureCompatible( const QgsFeature &feature, const QgsVectorLayer *layer );
+    static QgsFeatureList makeFeatureCompatible( const QgsFeature &feature, const QgsVectorLayer *layer, QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags() );
 
     /**
      * Converts input \a features to be compatible with the given \a layer.
@@ -271,17 +282,161 @@ class CORE_EXPORT QgsVectorLayerUtils
      * of input features.
      *
      * The following operations will be performed to convert the input features:
-     *  - convert single geometries to multi part
-     *  - drop additional attributes
-     *  - drop geometry if layer is geometry-less
-     *  - add missing attribute fields
-     *  - add back M/Z values (initialized to 0)
-     *  - drop Z/M
-     *  - convert multi part geometries to single part
+     *
+     * - convert single geometries to multi part
+     * - drop additional attributes
+     * - drop geometry if layer is geometry-less
+     * - add missing attribute fields
+     * - add back M/Z values (initialized to 0)
+     * - drop Z/M
+     * - convert multi part geometries to single part
+     *
+     * Optionally, \a sinkFlags can be specified to further refine the compatibility logic.
      *
      * \since QGIS 3.4
      */
-    static QgsFeatureList makeFeaturesCompatible( const QgsFeatureList &features, const QgsVectorLayer *layer );
+    static QgsFeatureList makeFeaturesCompatible( const QgsFeatureList &features, const QgsVectorLayer *layer, QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags() );
+
+    /**
+     * Tests whether a field is editable for a particular \a feature.
+     *
+     * \returns TRUE if the field at index \a fieldIndex from \a layer
+     * is editable, FALSE if the field is read only.
+     *
+     * \since QGIS 3.10
+     */
+    static bool fieldIsEditable( const QgsVectorLayer *layer, int fieldIndex, const QgsFeature &feature );
+
+    /**
+     * \returns TRUE if the field at index \a fieldIndex from \a layer
+     * is editable, FALSE if the field is read only.
+     *
+     * If this function returns TRUE then the editability of the field may still vary feature by
+     * feature. See fieldIsEditable() to determine this on a feature by feature basis.
+     *
+     * \since QGIS 3.18
+     */
+    static bool fieldIsReadOnly( const QgsVectorLayer *layer, int fieldIndex );
+
+    /**
+     * Returns TRUE if the editability of the field at index \a fieldIndex from \a layer may vary
+     * feature by feature.
+     *
+     * I.e. if the field is taken from a joined layer, the value may or may not be editable for any individual
+     * feature depending on the join's "upsert on edit" capabilities.
+     *
+     * \since QGIS 3.18
+     */
+    static bool fieldEditabilityDependsOnFeature( const QgsVectorLayer *layer, int fieldIndex );
+
+    /**
+      * Returns masks defined in labeling options of a layer.
+      * The returned type associates a labeling rule identifier to a set of layers that are masked given by their layer id,
+      * and a set of masked symbol layers if associated to each masked layers.
+      * \note Not available in Python bindings
+      * \since QGIS 3.12
+      */
+    static QHash<QString, QHash<QString, QSet<QgsSymbolLayerId>>> labelMasks( const QgsVectorLayer * ) SIP_SKIP;
+
+    /**
+     * Returns all masks that may be defined on symbol layers for a given vector layer.
+     * The hash key is a layer id.
+     * The hash value is the set of symbol layers masked in the key's layer.
+     * \note Not available in Python bindings
+     * \since QGIS 3.12
+     */
+    static QHash<QString, QSet<QgsSymbolLayerId>> symbolLayerMasks( const QgsVectorLayer * ) SIP_SKIP;
+
+    /**
+     * \returns a descriptive string for a \a feature, suitable for displaying to the user.
+     *         The definition is taken from the ``displayExpression`` property of \a layer.
+     * \since QGIS 3.12
+     */
+    static QString getFeatureDisplayString( const QgsVectorLayer *layer, const QgsFeature &feature );
+
+    /**
+     * Flags that can be used when determining cascaded features.
+     *
+     * \since QGIS 3.4
+     */
+    enum CascadedFeatureFlag
+    {
+      IgnoreAuxiliaryLayers = 1 << 1, //!< Ignore auxiliary layers
+    };
+    Q_DECLARE_FLAGS( CascadedFeatureFlags, CascadedFeatureFlag )
+
+    /**
+     * \returns TRUE if at least one feature of the \a fids on \a layer is connected as parent in at
+     * least one composition relation of the \a project or contains joins, where cascade delete is set.
+     * Details about cascading effects will be written to \a context.
+     * \since QGIS 3.14
+     */
+    static bool impactsCascadeFeatures( const QgsVectorLayer *layer, const QgsFeatureIds &fids, const QgsProject *project, QgsDuplicateFeatureContext &context SIP_OUT, QgsVectorLayerUtils::CascadedFeatureFlags flags = QgsVectorLayerUtils::CascadedFeatureFlags() );
+
+#ifndef SIP_RUN
+
+    /**
+     * Given a set of fields, attempts to pick the "most useful" field
+     * for user-friendly identification of features.
+     *
+     * For instance, if a field called "name" is present, this will be returned.
+     *
+     * Assumes that the user has organized the data with the more "interesting" field
+     * names first. As such, "name" would be selected before "oldname", "othername", etc.
+     *
+     * If no friendly identifier is found, the function will fallback to the
+     * first available.
+     *
+     * An optional boolean parameter can be used to determine whether the returned
+     * field name is a friendly identifier or not.
+     *
+     * \param fields list of fields to pick a friendly identifier from
+     * \param foundFriendly set to TRUE if the returned field name is a friendly identifier (since QGIS 3.22)
+     * \returns field name
+     * \since QGIS 3.18
+     */
+#else
+
+    /**
+     * Given a set of fields, attempts to pick the "most useful" field
+     * for user-friendly identification of features.
+     *
+     * For instance, if a field called "name" is present, this will be returned.
+     *
+     * Assumes that the user has organized the data with the more "interesting" field
+     * names first. As such, "name" would be selected before "oldname", "othername", etc.
+     *
+     * If no friendly identifier is found, the function will fallback to the
+     * first available.
+     *
+     * \param fields list of fields to pick a friendly identifier from
+     * \param foundFriendly set to TRUE if the returned field name is a friendly identifier
+     * \returns field name
+     * \since QGIS 3.22
+     */
+#endif
+    static QString guessFriendlyIdentifierField( const QgsFields &fields, bool *foundFriendly SIP_OUT = nullptr ) SIP_PYNAME( guessFriendlyIdentifierFieldV2 );
+
+#ifdef SIP_RUN
+
+    /**
+     * Given a set of fields, attempts to pick the "most useful" field
+     * for user-friendly identification of features.
+     *
+     * For instance, if a field called "name" is present, this will be returned.
+     *
+     * Assumes that the user has organized the data with the more "interesting" field
+     * names first. As such, "name" would be selected before "oldname", "othername", etc.
+     *
+     * If no friendly identifier is found, the function will fallback to the
+     * first available.
+     *
+     * \param fields list of fields to pick a friendly identifier from
+     * \returns field name
+     * \since QGIS 3.18
+     */
+    static QString guessFriendlyIdentifierField( const QgsFields &fields );
+#endif
 
 };
 
